using System;
using System.IO;
using System.Collections;
using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	public abstract class ASN1OctetString : DERObject
	{
		internal byte[]  m_string;

		/**
		* return an Octet String from a tagged object.
		*
		* @param obj the tagged object holding the object we want.
		* @param flagExplicit true if the object is meant to be explicitly
		*              tagged false otherwise.
		* @exception ArgumentException if the tagged object cannot
		*              be converted.
		*/
		public static ASN1OctetString getInstance(
			ASN1TaggedObject    obj,
			bool             flagExplicit)
		{
			return getInstance(obj.getObject());
		}
		
		/**
		* return an Octet String from the given object.
		*
		* @param obj the object we want converted.
		* @exception ArgumentException if the object cannot be converted.
		*/
		public static ASN1OctetString getInstance(Object  obj)
		{
			if ((obj == null) || (typeof(ASN1OctetString).IsInstanceOfType(obj)))
			{
				return (ASN1OctetString)obj;
			}

			if (typeof(ASN1TaggedObject).IsInstanceOfType(obj))
			{
				return getInstance(((ASN1TaggedObject)obj).getObject());
			}

			if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				ArrayList      v = new ArrayList();
				IEnumerator e = ((ASN1Sequence)obj).getObjects();

				while (e.MoveNext())
				{
					v.Add(e.Current);
				}

				return new BERConstructedOctetString(v);
			}

			throw new ArgumentException("illegal object in getInstance: " + obj.GetType().ToString());
		}

		/**
		* @param string the octets making up the octet string.
		*/
		public ASN1OctetString(
			byte[]  in_string)
		{
			this.m_string = in_string;
		}

		public ASN1OctetString(
			DEREncodable obj)
		{
			try
			{
				MemoryStream   bOut = new MemoryStream();
				DEROutputStream         dOut = new DEROutputStream(bOut);

				dOut.writeObject(obj);
				dOut.close();

				this.m_string = bOut.ToArray();
			}
			catch (IOException e)
			{
				throw new ArgumentException("Error processing object : " + e.ToString());
			}
		}

		public virtual byte[] getOctets()
		{
			return m_string;
		}

		public override int GetHashCode()
		{
			byte[]  b = this.getOctets();
			int     value = 0;

			for (int i = 0; i != b.Length; i++)
			{
				value ^= (b[i] & 0xff) << (i % 4);
			}

			return value;
		}


		public override bool Equals(
			Object  o)
		{
			if ((o == null) || !(typeof(DEROctetString).IsInstanceOfType(o)))
			{
				return false;
			}

			DEROctetString  other = (DEROctetString)o;

			byte[] b1 = other.getOctets();
			byte[] b2 = this.getOctets();

			if (b1.Length != b2.Length)
			{
				return false;
			}

			for (int i = 0; i != b1.Length; i++)
			{
				if (b1[i] != b2[i])
				{
					return false;
				}
			}

			return true;
		}

//		abstract void encode(DEROutputStream out)
//			throws IOException;

	}

}