using System;
using System.IO;
using System.Collections;
//using org.bouncycastle.io;

namespace org.bouncycastle.asn1
{
	/**
	* DER BMPString object.
	*/
	public class DERBMPString: DERObject, DERString
	{
		String  m_string;

		/**
		* return a BMP String from the given object.
		*
		* @param obj the object we want converted.
		* @exception ArgumentException if the object cannot be converted.
		*/
		public static DERBMPString getInstance(
			Object  obj)
		{
			if ((obj == null) || (typeof(DERBMPString).IsInstanceOfType(obj)))
			{
				return (DERBMPString)obj;
			}

			if (typeof(ASN1OctetString).IsInstanceOfType(obj))
			{
				return new DERBMPString(((ASN1OctetString)obj).getOctets());
			}

			if (typeof(ASN1TaggedObject).IsInstanceOfType(obj))
			{
				return getInstance(((ASN1TaggedObject)obj).getObject());
			}

			throw new ArgumentException("illegal object in getInstance: " +  obj.GetType().ToString());
		}

		/**
		* return a BMP String from a tagged object.
		*
		* @param obj the tagged object holding the object we want
		* @param flagExplicit true if the object is meant to be explicitly
		*              tagged false otherwise.
		* @exception ArgumentException if the tagged object cannot
		*              be converted.
		*/
		public static DERBMPString getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(obj.getObject());
		}
	    

		/**
		* basic constructor - byte encoded string.
		*/
		public DERBMPString(
			byte[]   in_string)
		{
			char[]  cs = new char[in_string.Length / 2];

			for (int i = 0; i != cs.Length; i++)
			{
				cs[i] = (char)((in_string[2 * i] << 8) | (in_string[2 * i + 1] & 0xff));
			}

			this.m_string = new String(cs);
		}

		/**
		* basic constructor
		*/
		public DERBMPString(String   in_string)
		{
			this.m_string = in_string;
		}

		public String getString()
		{
			return m_string;
		}

		public override int GetHashCode()
		{
			return this.getString().GetHashCode();
		}

		public override  bool Equals(Object  o)
		{
			if (!(typeof(DERBMPString).IsInstanceOfType(o)))
			{
				return false;
			}

			DERPrintableString  s = (DERPrintableString)o;

			return this.getString().Equals(s.getString());

		}

		public override void encode(DEROutputStream  outStream)
			//throws IOException
		{
			char[]  c = m_string.ToCharArray();
			byte[]  b = new byte[c.Length * 2];

			for (int i = 0; i != c.Length; i++)
			{
				b[2 * i] = (byte)(c[i] >> 8);
				b[2 * i + 1] = (byte)c[i];
			}

			outStream.writeEncoded(DERTags.BMP_STRING, b);
		}
	}

}