using System;
using System.Collections;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.pkcs
{
	public class PBES2Parameters: DEREncodable
	{
		private KeyDerivationFunc   func;
		private EncryptionScheme    scheme;

		public PBES2Parameters(
			ASN1Sequence  obj)
		{
			IEnumerator e = obj.getObjects();
			e.MoveNext();
			ASN1Sequence  funcSeq = (ASN1Sequence)e.Current;

			if (funcSeq.getObjectAt(0).Equals(PKCSObjectIdentifiers.id_PBKDF2))
			{
				func = new PBKDF2Params(funcSeq);
			}
			else
			{
				func = new KeyDerivationFunc(funcSeq);
			}
			
			e.MoveNext();
			scheme = new EncryptionScheme((ASN1Sequence)e.Current);
		}

		public KeyDerivationFunc getKeyDerivationFunc()
		{
			return func;
		}

		public EncryptionScheme getEncryptionScheme()
		{
			return scheme;
		}

		public DERObject getDERObject()
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();

			v.add(func);
			v.add(scheme);

			return new DERSequence(v);
		}
	}

}