
using System;
using System.Collections;
using org.bouncycastle.math;
using org.bouncycastle.asn1;

namespace org.bouncycastle.asn1.x509
{
	public class DSAParameter: DEREncodable
	{
		DERInteger      p, q, g;

		public static DSAParameter getInstance(
			ASN1TaggedObject obj,
			bool          flagExplicit)
		{
			return getInstance(ASN1Sequence.getInstance(obj, flagExplicit));
		}

		public static DSAParameter getInstance(
			Object obj)
		{
			if ((obj == null) || (typeof(DSAParameter).IsInstanceOfType(obj)))
			{
				return (DSAParameter)obj;
			}
	        
			if (typeof(ASN1Sequence).IsInstanceOfType(obj))
			{
				return new DSAParameter((ASN1Sequence)obj);
			}
	        
			throw new ArgumentException("Invalid DSAParameter: " + obj.GetType().ToString());
		}

		public DSAParameter(
			BigInteger  p,
			BigInteger  q,
			BigInteger  g)
		{
			this.p = new DERInteger(p);
			this.q = new DERInteger(q);
			this.g = new DERInteger(g);
		}

		public DSAParameter(
			ASN1Sequence  seq)
		{
			IEnumerator     e = seq.getObjects();
			e.MoveNext();
			p = (DERInteger)e.Current;
			e.MoveNext();
			q = (DERInteger)e.Current;
			e.MoveNext();
			g = (DERInteger)e.Current;
		}

		public BigInteger getP()
		{
			return p.getPositiveValue();
		}

		public BigInteger getQ()
		{
			return q.getPositiveValue();
		}

		public BigInteger getG()
		{
			return g.getPositiveValue();
		}

		public DERObject getDERObject()
		{
			ASN1EncodableVector  v = new ASN1EncodableVector();

			v.add(p);
			v.add(q);
			v.add(g);

			return new DERSequence(v);
		}
	}

}