
using System;
using System.IO;

namespace org.bouncycastle.io
{
	public class FilterInputStream : Stream 
	{
		protected Stream inStream;
		  
		protected FilterInputStream(Stream underlying) {
			inStream = underlying;
		}
		  
		public override int ReadByte() //throws IOException 
		{
			return inStream.ReadByte();
		}
		  
		public override int Read(byte[] b, int offset, int length) //throws IOException 
		{
			return inStream.Read(b, offset, length);
		}

		public override void Write(
			byte[] buffer,
			int offset,
			int count)
		{
			throw new NotSupportedException();
		}

		public override void WriteByte(byte val)
		{
			throw new NotSupportedException();
		}


		public override bool CanRead 
		{
			get { return true; 	}
		}

		public override bool CanWrite 
		{
			get { return false; 	}
		}

		public override bool CanSeek 
		{
			get { return inStream.CanSeek; 	}
		}

		public override long Length 
		{
			get { return inStream.Length; 	}
		}

		public override long Position 
		{
			get { return inStream.Position;   }
			set { inStream.Position = value;  }
		}

		public override void Close() //throws IOException 
		{
			inStream.Close();
		}

		public override  void Flush()
		{
			inStream.Flush();
		}

		public override long Seek(
			long offset,
			SeekOrigin origin)
		{
			return inStream.Seek(offset,origin);
		}

		public override void SetLength(long val)
		{
			inStream.SetLength(val);
		}





		public virtual int read() //throws IOException 
		{
			return inStream.ReadByte();
		}
		  
		public virtual int read(byte[] b) //throws IOException 
		{
			return Read(b, 0, b.Length);
		}
		  
		public virtual int read(byte[] b, int offset, int length) //throws IOException 
		{
			return inStream.Read(b, offset, length);
		}


//		public virtual long skip(long n) //throws IOException 
//		{
//			inStream.Position += n;
//		}
		  
		public virtual int available() //throws IOException 
		{
			return Convert.ToInt32(inStream.Length - inStream.Position);
		}
		  
		public virtual void close() //throws IOException 
		{
			inStream.Close();
		}
		  
//		new public virtual void mark(int readlimit) 
//		{
//			inStream.mark(readlimit);
//		}
//		  
//		new public virtual void reset() //throws IOException 
//		{
//			inStream.reset();
//		}
//		  
//		new public virtual bool markSupported() 
//		{
//			return inStream.markSupported();
//		}
	}

}