using System;

namespace org.bouncycastle.util.encoders
{
	/**
	* Converters for going from hex to binary and back. Note: this class assumes ASCII processing.
	*/
	public class HexTranslator : Translator
	{
		private static readonly byte[]   hexTable = 
			{ 
				(byte)'0', (byte)'1', (byte)'2', (byte)'3', (byte)'4', (byte)'5', (byte)'6', (byte)'7',
				(byte)'8', (byte)'9', (byte)'a', (byte)'b', (byte)'c', (byte)'d', (byte)'e', (byte)'f'
			};

		/**
		* size of the output block on encoding produced by getDecodedBlockSize()
		* bytes.
		*/
		public int getEncodedBlockSize()
		{
			return 2;
		}

		public int encode(
			byte[]  inBytes,
			int     inOff,
			int     length,
			byte[]  outBytes,
			int     outOff)
		{
			for (int i = 0, j = 0; i < length; i++, j += 2)
			{
				outBytes[outOff + j] = hexTable[(inBytes[inOff] >> 4) & 0x0f];
				outBytes[outOff + j + 1] = hexTable[inBytes[inOff] & 0x0f];

				inOff++;
			}

			return length * 2;
		}

		/**
		* size of the output block on decoding produced by getEncodedBlockSize()
		* bytes.
		*/
		public int getDecodedBlockSize()
		{
			return 1;
		}

		public int decode(
			byte[]  inBytes,
			int     inOff,
			int     length,
			byte[]  outBytes,
			int     outOff)
		{
			int halfLength = length / 2;
			byte left, right;
			for (int i = 0; i < halfLength; i++)
			{
				left  = inBytes[inOff + i * 2];
				right = inBytes[inOff + i * 2 + 1];
				
				if (left < (byte)'a')
				{
					outBytes[outOff] = (byte)((left - '0') << 4);
				}
				else
				{
					outBytes[outOff] = (byte)((left - 'a' + 10) << 4);
				}
				if (right < (byte)'a')
				{
					outBytes[outOff] += (byte)(right - '0');
				}
				else
				{
					outBytes[outOff] += (byte)(right - 'a' + 10);
				}

				outOff++;
			}

			return halfLength;
		}
	}

}